@@ -2,7 +2,7 @@
 #
 # AWL simulator - blocks
 #
-# Copyright 2012-2013 Michael Buesch <m@bues.ch>
+# Copyright 2012-2014 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,6 +60,8 @@ class BlockInterfaceField(object):
 			(ftype, self.name, str(self.dataType))
 
 class BlockInterface(object):
+	"""Code block interface (IN/OUT/IN_OUT/STAT/TEMP parameters) base class."""
+
 	# Data-types that must be passed "by-reference" to FCs/FBs.
 	callByRef_Types = (
 		AwlDataType.TYPE_TIMER,
@@ -120,6 +122,7 @@ class BlockInterface(object):
 		if field.name in self.fieldNameMap:
 			raise AwlSimError("Data structure field name '%s' is ambiguous." %\
 				field.name)
+
 		if field.fieldType == BlockInterfaceField.FTYPE_TEMP:
 			self.tempFieldCount += 1
 		elif field.fieldType == BlockInterfaceField.FTYPE_STAT:
@@ -153,12 +156,12 @@ class BlockInterface(object):
 		self.__addField(field)
 		self.fields_TEMP.append(field)
 
-	def __buildField(self, struct, field, isFirst):
+	def __buildField(self, cpu, struct, field, isFirst):
 		if isFirst:
-			struct.addFieldAligned(field.name,
+			struct.addFieldAligned(cpu, field.name,
 					       field.dataType, 2)
 		else:
-			struct.addFieldNaturallyAligned(field.name,
+			struct.addFieldNaturallyAligned(cpu, field.name,
 							field.dataType)
 
 	def __resolveMultiInstanceField(self, cpu, field):
@@ -222,13 +225,13 @@ class BlockInterface(object):
 			# Build instance-DB structure for the FB
 			self.struct = AwlStruct()
 			for i, field in enumerate(self.fields_IN):
-				self.__buildField(self.struct, field, i==0)
+				self.__buildField(cpu, self.struct, field, i==0)
 			for i, field in enumerate(self.fields_OUT):
-				self.__buildField(self.struct, field, i==0)
+				self.__buildField(cpu, self.struct, field, i==0)
 			for i, field in enumerate(self.fields_INOUT):
-				self.__buildField(self.struct, field, i==0)
+				self.__buildField(cpu, self.struct, field, i==0)
 			for i, field in enumerate(self.fields_STAT):
-				self.__buildField(self.struct, field, i==0)
+				self.__buildField(cpu, self.struct, field, i==0)
 		else:
 			# An FC does not have an instance-DB
 			assert(not self.fields_STAT) # No static data.
@@ -236,7 +239,7 @@ class BlockInterface(object):
 		# Build local-stack structure
 		self.tempStruct = AwlStruct()
 		for i, field in enumerate(self.fields_TEMP):
-			self.__buildField(self.tempStruct, field, i==0)
+			self.__buildField(cpu, self.tempStruct, field, i==0)
 		self.tempAllocation = self.tempStruct.getSize()
 		# If the OB-interface did not specify all automatic TEMP-fields,
 		# just force allocate them, so the lstack-allocator will not be confused.
@@ -383,6 +386,17 @@ class BlockInterface(object):
 		return '\n'.join(ret)
 
 class Block(object):
+	"""Base class for blocks (OBs, FCs, FBs, DBs, etc...)"""
+
+	def __init__(self, index):
+		self.index = index
+
+	def __repr__(self):
+		return "Block %d" % self.index
+
+class CodeBlock(Block):
+	"""Base class for code blocks (OBs, (S)FCs, (S)FBs)"""
+
 	# Simple and fast tests for checking block identity.
 	# These are partially overridden in the subclasses.
 	isOB		= False
@@ -391,9 +405,9 @@ class Block(object):
 	isSystemBlock	= False
 
 	def __init__(self, insns, index, interface):
+		Block.__init__(self, index)
 		self.insns = insns
 		self.labels = None
-		self.index = index
 		if insns:
 			self.labels = AwlLabel.resolveLabels(insns)
 		self.interface = interface
@@ -402,7 +416,7 @@ class Block(object):
 		pass
 
 	def __repr__(self):
-		return "Block %d" % self.index
+		return "CodeBlock %d" % self.index
 
 class OBInterface(BlockInterface):
 	startupTempAllocation = 20
@@ -419,11 +433,11 @@ class OBInterface(BlockInterface):
 	def addField_STAT(self, field):
 		raise AwlSimError("Static VAR not possible in an OB")
 
-class OB(Block):
+class OB(CodeBlock):
 	isOB = True
 
 	def __init__(self, insns, index):
-		Block.__init__(self, insns, index, OBInterface())
+		CodeBlock.__init__(self, insns, index, OBInterface())
 
 	def __repr__(self):
 		return "OB %d" % self.index
@@ -431,11 +445,11 @@ class OB(Block):
 class FBInterface(BlockInterface):
 	hasInstanceDB = True
 
-class FB(Block):
+class FB(CodeBlock):
 	isFB = True
 
 	def __init__(self, insns, index):
-		Block.__init__(self, insns, index, FBInterface())
+		CodeBlock.__init__(self, insns, index, FBInterface())
 
 	def __repr__(self):
 		return "FB %d" % self.index
@@ -444,11 +458,11 @@ class FCInterface(BlockInterface):
 	def addField_STAT(self, field):
 		raise AwlSimError("Static VAR not possible in an FC")
 
-class FC(Block):
+class FC(CodeBlock):
 	isFC = True
 
 	def __init__(self, insns, index):
-		Block.__init__(self, insns, index, FCInterface())
+		CodeBlock.__init__(self, insns, index, FCInterface())
 
 	def __repr__(self):
 		return "FC %d" % self.index
