@@ -20,13 +20,14 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
+
+from awlsim.common.datatypehelpers import *
 
 from awlsim.core.util import *
 from awlsim.core.operators import *
 from awlsim.core.optrans import *
 from awlsim.core.parser import *
-from awlsim.core.datatypehelpers import *
 
 
 class AwlInsn(object): #+cdef
