@@ -146,10 +146,74 @@ class RawAwlCodeBlock(RawAwlBlock):
 		return False
 
 class RawAwlDataIdent(object):
-	def __init__(self, name, indices=None):
-		self.name = name	# Name string of the variable
-		self.indices = indices	# Possible array indices (or None)
+	UPPERCASE	= 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+	LOWERCASE	= 'abcdefghijklmnopqrstuvwxyz'
+	NUMBERS		= '0123456789'
 
+	VALID_CHARS	= UPPERCASE + LOWERCASE + NUMBERS + '_'
+
+	@classmethod
+	def validateName(cls, name):
+		"""Check variable name against AWL naming rules.
+		Raises an exception in case of invalid name."""
+		# Check string length
+		if len(name) < 1:
+			raise AwlParserError("The variable name is too short")
+		if len(name) > 24:
+			raise AwlParserError("The variable name '%s' is "
+				"too long (max 24 characters)." %\
+				name)
+		# Only alphanumeric characters and underscores.
+		for c in name:
+			if c not in cls.VALID_CHARS:
+				raise AwlParserError("The variable name '%s' "
+					"contains invalid characters. "
+					"Only alphanumeric characters (a-z, A-Z, 0-9) "
+					"and underscores (_) are allowed." %\
+					name)
+		# First character must not be a number.
+		if name[0] in cls.NUMBERS:
+			raise AwlParserError("The first character of the "
+				"variable name '%s' must not be a number." %\
+				name)
+		# Last character must not be an underscore.
+		if name[-1] == '_':
+			raise AwlParserError("The last character of the "
+				"variable name '%s' must not be an underscore." %\
+				name)
+		# Consecutive underscores are not allowed.
+		if '__' in name:
+			raise AwlParserError("Consecutive underscores in "
+				"the variable name '%s' are not allowed." %\
+				name)
+		return name
+
+	def __init__(self, name, indices=None):
+		self.name = self.validateName(name)	# Name string of the variable
+		self.indices = indices			# Possible array indices (or None)
+
+	# Duplicate this ident
+	def dup(self):
+		return RawAwlDataIdent(self.name,
+				       self.indices[:] if self.indices else None)
+
+	# Increment the array indices of this ident by one.
+	# 'dimensions' is the array dimensions.
+	def advanceToNextArrayElement(self, dimensions):
+		assert(self.indices)
+		assert(len(self.indices) == len(dimensions))
+		self.indices[-1] += 1
+		for i in range(len(self.indices) - 1, -1, -1):
+			if self.indices[i] > dimensions[i][1]:
+				if i <= 0:
+					self.indices = [dim[0] for dim in dimensions]
+					break
+				self.indices[i] = dimensions[i][0]
+				self.indices[i - 1] += 1
+			else:
+				break
+
+	# == operator
 	def __eq__(self, other):
 		if self.name != other.name:
 			return False
@@ -158,6 +222,7 @@ class RawAwlDataIdent(object):
 				return False
 		return True
 
+	# != operator
 	def __ne__(self, other):
 		return not self.__eq__(other)
 
@@ -182,16 +247,16 @@ class RawAwlDataInit(object):
 		return self.getIdentString() + " := " + str(self.valueTokens)
 
 class RawAwlDataField(object):
-	def __init__(self, idents, typeTokens, dimensions=None, inits=None):
+	def __init__(self, idents, typeTokens, dimensions=None, defaultInits=None):
 		"""idents -> The identifications for the data field.
 		typeTokens -> List of tokens for the data type.
 		dimensions -> List of array dimensions, where each dimension is a
 		              tuple of (start, end). Or None, if this is not an array.
-		inits -> List of RawAwlDataInit()s"""
+		defaultInits -> List of RawAwlDataInit()s for use as 'startvalues'"""
 		self.idents = toList(idents)
 		self.typeTokens = typeTokens
 		self.dimensions = dimensions
-		self.inits = inits if inits else []	# List of RawAwlDataInit()s
+		self.defaultInits = defaultInits if defaultInits else [] # List of RawAwlDataInit()s
 
 	def getIdentString(self):
 		return ".".join(str(ident) for ident in self.idents)
@@ -210,10 +275,10 @@ class RawAwlDB(RawAwlBlock):
 		RawAwlBlock.__init__(self, tree, index)
 		self.fb = None
 
-		# Data fields and initializations.
-		# fieldInits are the inits from the DB init section.
-		# The inits from the DB declaration section are in fields[x].inits.
-		self.fields = []
+		# Data fields (RawAwlDataField) and initializations (RawAwlDataInit)
+		# fieldInits are the inits from the DB init section (_not_ declaration).
+		# The startup inits from the DB declaration section are in fields[x].inits.
+		self.fields = []	# List of RawAwlDataField()s
 		self.fieldInits = []	# List of RawAwlDataInit()s
 
 	def getField(self, idents):
@@ -225,33 +290,21 @@ class RawAwlDB(RawAwlBlock):
 	def addFieldInit(self, fieldInit):
 		self.fieldInits.append(fieldInit)
 
-	def getFieldInit(self, field):
-		"""Returns the RawAwlDataInit() for the specified RawAwlDataField()"""
-		for otherField, init in self.allFieldInits():
-			if field.idents == init.idents:
-				return init
-		return None
-
 	def allFieldInits(self):
-		"""Returns a list (generator) of all RawAwlDataInits()"""
-		# First all explicit field initializations
+		"""Returns a list (generator) of all 'actual-value' RawAwlDataInits()"""
 		for init in self.fieldInits:
 			yield self.getField(init.idents), init
-		# Then all field initializations from the declaration,
-		# that we did not initialize, yet.
-		for field in self.fields:
-			for init in field.inits:
-				for otherInit in self.fieldInits:
-					if init.idents == otherInit.idents:
-						# We already had an init for this field.
-						break
-				else:
-					# We did not have this init, yet.
-					yield field, init
 
 	def isInstanceDB(self):
 		return bool(self.fb)
 
+class RawAwlUDT(RawAwlBlock):
+	def __init__(self, tree, index):
+		RawAwlBlock.__init__(self, tree, index)
+
+		# The data fields of this UDT
+		self.fields = []	# List of RawAwlDataField()s
+
 class RawAwlOB(RawAwlCodeBlock):
 	def __init__(self, tree, index):
 		RawAwlCodeBlock.__init__(self, tree, index)
@@ -267,10 +320,11 @@ class RawAwlFC(RawAwlCodeBlock):
 
 class AwlParseTree(object):
 	def __init__(self):
-		self.dbs = {}
-		self.fbs = {}
-		self.fcs = {}
-		self.obs = {}
+		self.dbs = {}	# DBs (dict of 'OB-name : RawAwlDB()')
+		self.fbs = {}	# FBs (dict of 'FB-name : RawAwlFB()')
+		self.fcs = {}	# FCs (dict of 'FC-name : RawAwlFC()')
+		self.obs = {}	# OBs (dict of 'OB-name : RawAwlOB()')
+		self.udts = {}	# UDTs (dict of 'UDT-name : RawAwlUDT()')
 
 		self.curBlock = None
 
@@ -302,6 +356,8 @@ class AwlParser(object):
 	STATE_IN_OB_HDR_VARTEMP		= EnumGen.item
 	STATE_IN_OB_HDR_ATTR		= EnumGen.item
 	STATE_IN_OB			= EnumGen.item
+	STATE_IN_UDT_HDR		= EnumGen.item
+	STATE_IN_UDT_HDR_STRUCT		= EnumGen.item
 	EnumGen.end
 
 	TEXT_ENCODING = "latin_1"
@@ -361,6 +417,23 @@ class AwlParser(object):
 		return self.__inAnyHeader() or\
 		       self.state == self.STATE_GLOBAL
 
+	def __inVariableSection(self):
+		if self.flatLayout:
+			return False
+		return self.state in (self.STATE_IN_DB_HDR_STRUCT,
+				      self.STATE_IN_DB,
+				      self.STATE_IN_FB_HDR_VAR,
+				      self.STATE_IN_FB_HDR_VARIN,
+				      self.STATE_IN_FB_HDR_VAROUT,
+				      self.STATE_IN_FB_HDR_VARINOUT,
+				      self.STATE_IN_FB_HDR_VARTEMP,
+				      self.STATE_IN_FC_HDR_VARIN,
+				      self.STATE_IN_FC_HDR_VAROUT,
+				      self.STATE_IN_FC_HDR_VARINOUT,
+				      self.STATE_IN_FC_HDR_VARTEMP,
+				      self.STATE_IN_OB_HDR_VARTEMP,
+				      self.STATE_IN_UDT_HDR_STRUCT)
+
 	def __tokenize(self, data, sourceId, sourceName):
 		self.reset()
 		self.tree.sourceId = sourceId
@@ -449,7 +522,20 @@ class AwlParser(object):
 					t.addToken(c)
 					continue
 			if not t.inParens:
-				if c in ('\n', ';'):
+				# Check whether we have tokenized a whole statement.
+				# In variable sections, this is if we hit a semicolon or
+				# END_STRUCT, END_VAR or END_DATA_BLOCK.
+				# In code, this is if we hit a semicolon or newline (for convenience).
+				wholeStatementOk = False
+				if self.__inVariableSection():
+					if c.isspace() and\
+					   t.curToken.upper() in ("END_STRUCT", "END_VAR", "END_DATA_BLOCK"):
+						wholeStatementOk = True
+					if c == ';':
+						wholeStatementOk = True
+				elif c in (';', '\n'):
+					wholeStatementOk = True
+				if wholeStatementOk:
 					self.__parseTokens(t)
 					continue
 			if c.isspace():
@@ -516,6 +602,10 @@ class AwlParser(object):
 			self.__parseTokens_ob_hdr_attr(tokenizerState)
 		elif self.state == self.STATE_IN_OB:
 			self.__parseTokens_ob(tokenizerState)
+		elif self.state == self.STATE_IN_UDT_HDR:
+			self.__parseTokens_udt_hdr(tokenizerState)
+		elif self.state == self.STATE_IN_UDT_HDR_STRUCT:
+			self.__parseTokens_udt_hdr_struct(tokenizerState)
 		else:
 			assert(0)
 
@@ -620,11 +710,31 @@ class AwlParser(object):
 						"of OB %s" % str(obNumber))
 				self.tree.obs[obNumber] = self.tree.curBlock
 				return
+			if t.tokens[0].upper() == "TYPE":
+				self.__setState(self.STATE_IN_UDT_HDR)
+				if t.tokens[1].startswith('"') and\
+				   t.tokens[1].endswith('"'):
+					# UDT name is symbolic
+					udtNumber = t.tokens[1][1:-1]
+				else:
+					# UDT name is absolute
+					if t.tokens[1].upper() != "UDT":
+						raise AwlParserError("Invalid UDT name")
+					try:
+						udtNumber = int(t.tokens[2], 10)
+					except ValueError:
+						raise AwlParserError("Invalid UDT number")
+				self.tree.curBlock = RawAwlUDT(self.tree, udtNumber)
+				if udtNumber in self.tree.udts:
+					raise AwlParserError("Multiple definitions "
+						"of UDT %s" % str(udtNumber))
+				self.tree.udts[udtNumber] = self.tree.curBlock
+				return
 		except IndexError as e:
 			raise AwlParserError("Missing token")
 		except ValueError as e:
 			raise AwlParserError("Invalid value")
-		raise AwlParserError("Unknown statement")
+		raise AwlParserError("Unknown statement: %s" % " ".join(t.tokens))
 
 	def __parseInstruction(self, t):
 		insn = RawAwlInsn(self.tree.curBlock)
@@ -670,17 +780,70 @@ class AwlParser(object):
 			fbSymbol = name[1:-1]
 			self.tree.curBlock.fb = RawAwlDB.FBRef(fbSymbol = fbSymbol)
 		else:
-			raise AwlParserError("In DB header: Unknown token: %s" % name)
+			raise AwlParserError("In DB header: Unknown token: %s\n"\
+					     "Maybe missing semicolon in preceding lines?"\
+					     % name)
 
-	def __parseArrayInitializer(self, name, initsList, tokens):
+	def __parseArrayInitializer(self, name, dimensions, initsList, tokens):
 		"""Parse an ARRAY initializer. That is either of:
 		1, 2, 3, 4
 		4 (1, 2, 3, 4)
 		or similar.
 		name -> The name string of the variable.
+		dimensions -> The array dimensions.
 		initsList -> The result list. Each element is a RawAwlDataInit().
 		tokens -> The tokens to parse."""
-		pass#TODO
+
+		ident = RawAwlDataIdent(name,
+					[dim[0] for dim in dimensions])
+		repeatCount = None
+		repeatValues = []
+		valueTokens = []
+
+		def addInit(valTokens):
+			init = RawAwlDataInit(ident.dup(), valTokens)
+			initsList.append(init)
+			ident.advanceToNextArrayElement(dimensions)
+
+		while tokens:
+			if tokens[0] == ',':
+				if repeatCount is None:
+					addInit(valueTokens)
+				else:
+					repeatValues.append(valueTokens)
+				valueTokens = []
+			elif tokens[0] == '(':
+				if repeatCount is not None:
+					raise AwlParserError("Nested data initialization "
+						"repetitions are not allowed.")
+				# Starting a repetition. The current valueTokens
+				# is the repeat count.
+				try:
+					if len(valueTokens) != 1:
+						raise ValueError
+					repeatCount = int(valueTokens[0])
+				except ValueError:
+					raise AwlParserError("Invalid repeat count.")
+				if repeatCount <= 0 or repeatCount > 0x7FFF:
+					raise AwlParserError("Repeat count is out of range. "
+						"Count must be between 1 and 32767.")
+				valueTokens = []
+			elif tokens[0] == ')':
+				repeatValues.append(valueTokens)
+				valueTokens = []
+				if repeatCount is None:
+					raise AwlParserError("Too many closing parenthesis.")
+				# Add the repeated values to the init list.
+				for i in range(repeatCount):
+					for valToks in repeatValues:
+						addInit(valToks)
+				repeatCount = None
+				repeatValues = []
+			else:
+				valueTokens.append(tokens[0])
+			tokens = tokens[1:]
+		if valueTokens:
+			addInit(valueTokens)
 
 	def __parse_var_generic(self, t, varList,
 				endToken,
@@ -747,6 +910,7 @@ class AwlParser(object):
 				# Parse the ARRAY initializer (:= ...)
 				initTokens = t.tokens[assignIdx+1:]
 				self.__parseArrayInitializer(name,
+							     dimensions,
 							     initsList,
 							     initTokens)
 		else:
@@ -762,11 +926,12 @@ class AwlParser(object):
 				name = t.tokens[0]
 				type = t.tokens[colonIdx+1:]
 			else:
-				raise AwlParserError("In variable section: Unknown tokens")
+				raise AwlParserError("In variable section: Unknown tokens.\n"\
+						     "Maybe missing semicolon in preceding lines?")
 		field = RawAwlDataField(idents = RawAwlDataIdent(name),
 					typeTokens = type,
 					dimensions = dimensions,
-					inits = initsList)
+					defaultInits = initsList)
 		varList.append(field)
 
 		return True
@@ -825,7 +990,8 @@ class AwlParser(object):
 			db.addFieldInit(RawAwlDataInit(RawAwlDataIdent(name),
 						       valueTokens))
 		else:
-			raise AwlParserError("In DB: Unknown tokens")
+			raise AwlParserError("In DB: Unknown tokens.\n"\
+					     "Maybe missing semicolon in preceding lines?")
 
 	def __parseTokens_fb_hdr(self, t):
 		name = t.tokens[0].upper()
@@ -848,7 +1014,9 @@ class AwlParser(object):
 			if "}" not in t.tokens:
 				self.__setState(self.STATE_IN_FB_HDR_ATTR)
 		else:
-			raise AwlParserError("In FB: Unknown token: %s" % name)
+			raise AwlParserError("In FB: Unknown token: %s\n"\
+					     "Maybe missing semicolon in preceding lines?"\
+					     % name)
 
 	def __parseTokens_fb_hdr_var(self, t):
 		if not self.__parse_var_generic(t,
@@ -915,7 +1083,9 @@ class AwlParser(object):
 			if "}" not in t.tokens:
 				self.__setState(self.STATE_IN_FC_HDR_ATTR)
 		else:
-			raise AwlParserError("In FC header: Unknown token: %s" % name)
+			raise AwlParserError("In FC header: Unknown token: %s\n"\
+					     "Maybe missing semicolon in preceding lines?"\
+					     % name)
 
 	def __parseTokens_fc_hdr_varin(self, t):
 		if not self.__parse_var_generic(t,
@@ -973,7 +1143,9 @@ class AwlParser(object):
 			if "}" not in t.tokens:
 				self.__setState(self.STATE_IN_OB_HDR_ATTR)
 		else:
-			raise AwlParserError("In OB header: Unknown token: %s" % name)
+			raise AwlParserError("In OB header: Unknown token: %s\n"\
+					     "Maybe missing semicolon in preceding lines?"\
+					     % name)
 
 	def __parseTokens_ob_hdr_vartemp(self, t):
 		if not self.__parse_var_generic(t,
@@ -997,6 +1169,25 @@ class AwlParser(object):
 		insn = self.__parseInstruction(t)
 		self.tree.curBlock.insns.append(insn)
 
+	def __parseTokens_udt_hdr(self, t):
+		name = t.tokens[0].upper()
+		if name == "END_TYPE":
+			self.__setState(self.STATE_GLOBAL)
+		elif name in ("TITLE", "AUTHOR", "FAMILY", "NAME", "VERSION"):
+			self.tree.curBlock.addDescriptor(t.tokens)
+		elif name == "STRUCT":
+			self.__setState(self.STATE_IN_UDT_HDR_STRUCT)
+		else:
+			raise AwlParserError("In UDT: Unknown token: %s\n"\
+					     "Maybe missing semicolon in preceding lines?"\
+					     % name)
+
+	def __parseTokens_udt_hdr_struct(self, t):
+		if not self.__parse_var_generic(t,
+				varList = self.tree.curBlock.fields,
+				endToken = "END_STRUCT"):
+			self.__setState(self.STATE_IN_UDT_HDR)
+
 	def parseSource(self, awlSource):
 		"""Parse an AWL source.
 		awlSource is an AwlSource instance."""
@@ -1017,7 +1208,9 @@ class AwlParser(object):
 				  re.DOTALL | re.MULTILINE)
 		haveOB = re.match(r'.*^\s*ORGANIZATION_BLOCK\s+.*', sourceText,
 				  re.DOTALL | re.MULTILINE)
-		return not haveDB and not haveFB and not haveFC and not haveOB
+		haveUDT = re.match(r'.*^\s*TYPE\s+.*', sourceText,
+				   re.DOTALL | re.MULTILINE)
+		return not haveDB and not haveFB and not haveFC and not haveOB and not haveUDT
 
 	def parseData(self, dataBytes, sourceId=None, sourceName=None):
 		try:
