@@ -20,10 +20,11 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
+
+from awlsim.common.datatypehelpers import *
 
 from awlsim.core.util import *
-from awlsim.core.datatypehelpers import *
 
 
 def _seconds_to_s5t_tb10ms(seconds):
