@@ -59,6 +59,7 @@ class AwlTranslator(object):
 		return insns
 
 	def __translateInterfaceField(self, rawVar):
+		#TODO use rawFieldTranslate
 		dtype = AwlDataType.makeByName(rawVar.typeTokens, rawVar.dimensions)
 		assert(len(rawVar.idents) == 1) #TODO no structs, yet
 		field = BlockInterfaceField(name = rawVar.idents[0].name,
@@ -90,33 +91,61 @@ class AwlTranslator(object):
 			block.interface.addField_TEMP(self.__translateInterfaceField(rawVar))
 		return block
 
+	@classmethod
+	def rawFieldTranslate(cls, rawField):
+		name = rawField.getIdentString()
+		dataType = AwlDataType.makeByName(rawField.typeTokens,
+						  rawField.dimensions)
+		if rawField.defaultInits:
+			# Translate the initialization values and
+			# put them into a ByteArray.
+			initBytes = ByteArray(intDivRoundUp(dataType.width, 8))
+			if dataType.type == AwlDataType.TYPE_ARRAY:
+				for rawDataInit in rawField.defaultInits:
+					value = dataType.parseMatchingImmediate(rawDataInit.valueTokens)
+					linArrayIndex = dataType.arrayIndicesCollapse(
+						rawDataInit.idents[-1].indices)
+					offset = AwlOffset.fromBitOffset(linArrayIndex *
+									 dataType.children[0].width)
+					try:
+						initBytes.store(offset, dataType.children[0].width,
+								value)
+					except AwlSimError as e:
+						raise AwlSimError("Data field '%s' initialization "
+							"is out of range." % str(rawField))
+			else:
+				assert(len(rawField.defaultInits) == 1)
+				value = dataType.parseMatchingImmediate(rawField.defaultInits[0].valueTokens)
+				try:
+					initBytes.store(AwlOffset(), dataType.width, value)
+				except AwlSimError as e:
+					raise AwlSimError("Data field '%s' initialization "
+						"is out of range." % str(rawField))
+		else:
+			initBytes = None
+		return name, dataType, initBytes
+
 	# Initialize a DB (global or instance) data field from a raw data-init.
 	def __initDBField(self, db, dataType, rawDataInit):
+		#TODO no structs, yet
+		fieldName = rawDataInit.idents[-1].name
 		if dataType.type == AwlDataType.TYPE_ARRAY:
 			index = dataType.arrayIndicesCollapse(rawDataInit.idents[-1].indices)
 		else:
 			index = None
 		value = dataType.parseMatchingImmediate(rawDataInit.valueTokens)
-		db.structInstance.setFieldDataByName(rawDataInit.idents[-1].name,
-						     index, value)
+		db.structInstance.setFieldDataByName(fieldName, index, value)
 
 	def __translateGlobalDB(self, rawDB):
 		db = DB(rawDB.index, None)
 		# Create the data structure fields
 		for field in rawDB.fields:
-			assert(len(field.idents) == 1) #TODO no structs, yet
-			if not rawDB.getFieldInit(field):
-				raise AwlSimError(
-					"DB %d declares field '%s', "
-					"but does not initialize." %\
-					(rawDB.index, field.idents[0].name))
-			dtype = AwlDataType.makeByName(field.typeTokens,
-						       field.dimensions)
-			db.struct.addFieldNaturallyAligned(field.idents[0].name,
-							   dtype)
+			name, dataType, initBytes = self.rawFieldTranslate(field)
+			db.struct.addFieldNaturallyAligned(self.cpu, name,
+							   dataType, initBytes)
 		# Allocate the data structure fields
 		db.allocate()
-		# Initialize the data structure fields
+		# Assign data structure initializations.
 		for field, init in rawDB.allFieldInits():
 			if not field:
 				raise AwlSimError(
@@ -176,7 +205,7 @@ class AwlTranslator(object):
 		return db
 
 	def translateDB(self, rawDB):
-		if rawDB.index < 0 or rawDB.index > 0xFFFF:
+		if rawDB.index < 1 or rawDB.index > 0xFFFF:
 			raise AwlSimError("DB number %d is invalid" % rawDB.index)
 		if rawDB.isInstanceDB():
 			return self.__translateInstanceDB(rawDB)
