@@ -2,10 +2,11 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 
 import awlsim.cython_helper as __cython
 
-if __cython.shouldUseCython():
+if __cython.shouldUseCython():						#@nocy
+#if True:								#@cy
 	try:
 		from awlsim_cython.coreserver.all_modules import *	#<no-cython-patch
 	except ImportError as e:
 		__cython.cythonImportError("coreserver", str(e))
-if not __cython.shouldUseCython():
-	from awlsim.coreserver.all_modules import *			#<no-cython-patch
+if not __cython.shouldUseCython():					#@nocy
+	from awlsim.coreserver.all_modules import *			#@nocy
