@@ -27,7 +27,7 @@ from awlsim.gui.symtabwidget import *
 from awlsim.gui.util import *
 
 
-class SourceTabCorner(QWidget):
+class SourceTabContextMenu(QMenu):
 	# Signal: Add new source
 	add = Signal()
 	# Signal: Delete current source
@@ -42,26 +42,20 @@ class SourceTabCorner(QWidget):
 	export = Signal()
 
 	def __init__(self, itemName, parent=None):
-		QWidget.__init__(self, parent)
-		self.setLayout(QGridLayout())
-		self.layout().setContentsMargins(QMargins(3, 0, 0, 0))
+		QMenu.__init__(self, parent)
 
 		self.itemName = itemName
 
-		self.menu = QMenu(self)
-		self.menu.addAction("&Add %s" % itemName, self.__add)
-		self.menu.addAction("&Delete %s..." % itemName, self.__delete)
-		self.menu.addAction("&Rename %s..." % itemName, self.__rename)
-		self.menu.addSeparator()
-		self.menu.addAction("&Import %s..." % itemName, self.__import)
-		self.menu.addAction("&Export %s..." % itemName, self.__export)
-		self.__integrateAction = self.menu.addAction("&Integrate %s into project..." % itemName,
-							     self.__integrate)
-		self.showIntegrateButton(False)
+		self.addAction("&Add %s" % itemName, self.__add)
+		self.addAction("&Delete %s..." % itemName, self.__delete)
+		self.addAction("&Rename %s..." % itemName, self.__rename)
+		self.addSeparator()
+		self.addAction("&Import %s..." % itemName, self.__import)
+		self.addAction("&Export %s..." % itemName, self.__export)
+		self.__integrateAction = self.addAction("&Integrate %s into project..." % itemName,
+							self.__integrate)
 
-		self.menuButton = QPushButton("&" + itemName[0].upper() + itemName[1:], self)
-		self.menuButton.setMenu(self.menu)
-		self.layout().addWidget(self.menuButton, 0, 0)
+		self.showIntegrateButton(False)
 
 	def __add(self):
 		self.add.emit()
@@ -72,6 +66,12 @@ class SourceTabCorner(QWidget):
 	def __rename(self):
 		self.rename.emit()
 
+	def __import(self):
+		self.import_.emit()
+
+	def __export(self):
+		self.export.emit()
+
 	def __integrate(self):
 		res = QMessageBox.question(self,
 			"Integrate current %s" % self.itemName,
@@ -86,11 +86,15 @@ class SourceTabCorner(QWidget):
 	def showIntegrateButton(self, show=True):
 		self.__integrateAction.setVisible(show)
 
-	def __import(self):
-		self.import_.emit()
+class SourceTabCorner(QWidget):
+	def __init__(self, itemName, contextMenu, parent=None):
+		QWidget.__init__(self, parent)
+		self.setLayout(QGridLayout())
+		self.layout().setContentsMargins(QMargins(3, 0, 0, 0))
 
-	def __export(self):
-		self.export.emit()
+		self.menuButton = QPushButton("&" + itemName[0].upper() + itemName[1:], self)
+		self.menuButton.setMenu(contextMenu)
+		self.layout().addWidget(self.menuButton, 0, 0)
 
 class SourceTabWidget(QTabWidget):
 	"Abstract source tab-widget"
@@ -102,11 +106,13 @@ class SourceTabWidget(QTabWidget):
 		QTabWidget.__init__(self, parent)
 		self.itemName = itemName
 
+		self.contextMenu = SourceTabContextMenu(itemName, self)
+
 		self.setMovable(True)
-		self.actionButton = SourceTabCorner(itemName, self)
+		self.actionButton = SourceTabCorner(itemName, self.contextMenu, self)
 		self.setCornerWidget(self.actionButton, Qt.TopRightCorner)
 
-		self.actionButton.integrate.connect(self.integrateSource)
+		self.contextMenu.integrate.connect(self.integrateSource)
 		self.currentChanged.connect(self.__currentChanged)
 		self.tabBar().tabMoved.connect(self.__tabMoved)
 
@@ -123,12 +129,12 @@ class SourceTabWidget(QTabWidget):
 		curWidget = self.currentWidget()
 		showIntegrate = False
 		if curWidget:
-			showIntegrate = curWidget.getSourceRef().isFileBacked()
-		self.actionButton.showIntegrateButton(showIntegrate)
+			showIntegrate = curWidget.getSource().isFileBacked()
+		self.contextMenu.showIntegrateButton(showIntegrate)
 
 	def updateTabTexts(self):
 		for i in range(self.count()):
-			self.setTabText(i, self.widget(i).getSourceRef().name)
+			self.setTabText(i, self.widget(i).getSource().name)
 		self.sourceChanged.emit()
 
 	def allTabWidgets(self):
@@ -145,7 +151,7 @@ class SourceTabWidget(QTabWidget):
 
 	def getSources(self):
 		"Returns a list of sources"
-		return [ w.getFullSource() for w in self.allTabWidgets() ]
+		return [ w.getSource() for w in self.allTabWidgets() ]
 
 	def setSources(self, sources):
 		raise NotImplementedError
@@ -153,10 +159,17 @@ class SourceTabWidget(QTabWidget):
 	def integrateSource(self):
 		curWidget = self.currentWidget()
 		if curWidget:
-			curWidget.getSourceRef().forceNonFileBacked(self.actionButton.itemName)
+			curWidget.getSource().forceNonFileBacked(self.contextMenu.itemName)
 			self.updateActionMenu()
 			self.updateTabTexts()
 
+	def contextMenuEvent(self, ev):
+		QTabWidget.contextMenuEvent(self, ev)
+		tabBar = self.tabBar()
+		if tabBar.geometry().contains(tabBar.mapFrom(self, ev.pos())):
+			# Tab context menu was requested.
+			self.contextMenu.exec_(self.mapToGlobal(ev.pos()))
+
 class AwlSourceTabWidget(SourceTabWidget):
 	"AWL source tab-widget"
 
@@ -169,12 +182,12 @@ class AwlSourceTabWidget(SourceTabWidget):
 
 		self.reset()
 
-		self.actionButton.add.connect(self.addEditWidget)
-		self.actionButton.delete.connect(self.deleteCurrent)
-		self.actionButton.rename.connect(self.renameCurrent)
-		self.actionButton.export.connect(self.exportCurrent)
+		self.contextMenu.add.connect(self.addEditWidget)
+		self.contextMenu.delete.connect(self.deleteCurrent)
+		self.contextMenu.rename.connect(self.renameCurrent)
+		self.contextMenu.export.connect(self.exportCurrent)
+		self.contextMenu.import_.connect(self.importSource)
 		self.currentChanged.connect(self.__currentChanged)
-		self.actionButton.import_.connect(self.importSource)
 
 	def reset(self):
 		SourceTabWidget.reset(self)
@@ -185,7 +198,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 		editWidget = self.currentWidget()
 		if editWidget:
 			fromLine, toLine = editWidget.getVisibleLineRange()
-			source = editWidget.getSourceRef()
+			source = editWidget.getSource()
 			self.visibleLinesChanged.emit(source, fromLine, toLine)
 		else:
 			self.visibleLinesChanged.emit(None, -1, -1)
@@ -232,7 +245,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 		editWidget = EditWidget(self)
 		editWidget.codeChanged.connect(self.sourceChanged)
 		editWidget.visibleRangeChanged.connect(self.__emitVisibleLinesSignal)
-		index = self.addTab(editWidget, editWidget.getSourceRef().name)
+		index = self.addTab(editWidget, editWidget.getSource().name)
 		self.setCurrentIndex(index)
 		self.updateActionMenu()
 		self.sourceChanged.emit()
@@ -261,7 +274,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 					text)
 			if ok and newText != text:
 				editWidget = self.widget(index)
-				source = editWidget.getSourceRef()
+				source = editWidget.getSource()
 				source.name = newText
 				self.updateTabTexts()
 
@@ -269,7 +282,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 		editWidget = self.currentWidget()
 		if not editWidget:
 			return
-		source = editWidget.getFullSource()
+		source = editWidget.getSource()
 		if not source:
 			return
 		fn, fil = QFileDialog.getSaveFileName(self,
@@ -293,8 +306,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 			"All files (*)")
 		if not fn:
 			return
-		source = AwlSource.fromFile(AwlSource.newIdentNr(),
-					    "Imported source",
+		source = AwlSource.fromFile("Imported source",
 					    fn)
 		index, editWidget = self.addEditWidget()
 		editWidget.setSource(source)
@@ -314,11 +326,11 @@ class SymSourceTabWidget(SourceTabWidget):
 
 		self.reset()
 
-		self.actionButton.add.connect(self.addSymTable)
-		self.actionButton.delete.connect(self.deleteCurrent)
-		self.actionButton.rename.connect(self.renameCurrent)
-		self.actionButton.export.connect(self.exportCurrent)
-		self.actionButton.import_.connect(self.importSource)
+		self.contextMenu.add.connect(self.addSymTable)
+		self.contextMenu.delete.connect(self.deleteCurrent)
+		self.contextMenu.rename.connect(self.renameCurrent)
+		self.contextMenu.export.connect(self.exportCurrent)
+		self.contextMenu.import_.connect(self.importSource)
 
 	def reset(self):
 		SourceTabWidget.reset(self)
@@ -340,7 +352,7 @@ class SymSourceTabWidget(SourceTabWidget):
 		symTabView = SymTabView(self)
 		symTabView.setSymTab(SymbolTable())
 		symTabView.model().sourceChanged.connect(self.sourceChanged)
-		index = self.addTab(symTabView, symTabView.model().getSourceRef().name)
+		index = self.addTab(symTabView, symTabView.model().getSource().name)
 		self.setCurrentIndex(index)
 		self.updateActionMenu()
 		self.sourceChanged.emit()
@@ -369,7 +381,7 @@ class SymSourceTabWidget(SourceTabWidget):
 					text)
 			if ok and newText != text:
 				symTabView = self.widget(index)
-				source = symTabView.getSourceRef()
+				source = symTabView.getSource()
 				source.name = newText
 				self.updateTabTexts()
 
@@ -377,7 +389,7 @@ class SymSourceTabWidget(SourceTabWidget):
 		symTabView = self.currentWidget()
 		if not symTabView:
 			return
-		source = symTabView.getFullSource()
+		source = symTabView.getSource()
 		if not source:
 			return
 		fn, fil = QFileDialog.getSaveFileName(self,
@@ -401,8 +413,7 @@ class SymSourceTabWidget(SourceTabWidget):
 			"All files (*)")
 		if not fn:
 			return
-		source = SymTabSource.fromFile(SymTabSource.newIdentNr(),
-					       "Imported symbol table",
+		source = SymTabSource.fromFile("Imported symbol table",
 					       fn)
 		index, symTabView = self.addSymTable()
 		symTabView.setSource(source)
