@@ -20,12 +20,13 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
+
+from awlsim.common import *
+from awlsim.coreclient import *
+import awlsim.cython_helper
 
 import sys
-from awlsim.core import *
-from awlsim.coreserver import *
-import awlsim.cython_helper
 
 if isPyPy or isJython:
 	# PySide does not work on PyPy or Jython, yet.
