@@ -20,11 +20,12 @@
 #
 
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
+
+from awlsim.common.datatypehelpers import *
 
 from awlsim.core.hardware import *
 from awlsim.core.util import *
-from awlsim.core.datatypehelpers import *
 
 
 class HardwareInterface(AbstractHardwareInterface):
